@@ -7,7 +7,7 @@ __docformat__ = 'restructuredtext en'
 lxml based OPF parser.
 '''
 
-import re, sys, unittest, functools, os, mimetypes, uuid, glob, cStringIO
+import re, sys, unittest, functools, os, mimetypes, uuid, glob, cStringIO, json
 from urllib import unquote
 from urlparse import urlparse
 
@@ -16,11 +16,13 @@ from lxml import etree
 from calibre.ebooks.chardet import xml_to_unicode
 from calibre.constants import __appname__, __version__, filesystem_encoding
 from calibre.ebooks.metadata.toc import TOC
-from calibre.ebooks.metadata import MetaInformation, string_to_authors
+from calibre.ebooks.metadata import string_to_authors, MetaInformation
+from calibre.ebooks.metadata.book.base import Metadata
 from calibre.utils.date import parse_date, isoformat
 from calibre.utils.localization import get_lang
+from calibre import prints
 
-class Resource(object):
+class Resource(object): # {{{
     '''
     Represents a resource (usually a file on the filesystem or a URL pointing
     to the web. Such resources are commonly referred to in OPF files.
@@ -101,8 +103,9 @@ class Resource(object):
     def __repr__(self):
         return 'Resource(%s, %s)'%(repr(self.path), repr(self.href()))
 
+# }}}
 
-class ResourceCollection(object):
+class ResourceCollection(object): # {{{
 
     def __init__(self):
         self._resources = []
@@ -153,10 +156,9 @@ class ResourceCollection(object):
         for res in self:
             res.set_basedir(path)
 
+# }}}
 
-
-
-class ManifestItem(Resource):
+class ManifestItem(Resource): # {{{
 
     @staticmethod
     def from_opf_manifest_item(item, basedir):
@@ -194,8 +196,9 @@ class ManifestItem(Resource):
             return self.media_type
         raise IndexError('%d out of bounds.'%index)
 
+# }}}
 
-class Manifest(ResourceCollection):
+class Manifest(ResourceCollection): # {{{
 
     @staticmethod
     def from_opf_manifest_element(items, dir):
@@ -262,7 +265,9 @@ class Manifest(ResourceCollection):
             if i.id == id:
                 return i.mime_type
 
-class Spine(ResourceCollection):
+# }}}
+
+class Spine(ResourceCollection): # {{{
 
     class Item(Resource):
 
@@ -334,7 +339,9 @@ class Spine(ResourceCollection):
         for i in self:
             yield i.path
 
-class Guide(ResourceCollection):
+# }}}
+
+class Guide(ResourceCollection): # {{{
 
     class Reference(Resource):
 
@@ -371,14 +378,17 @@ class Guide(ResourceCollection):
             self[-1].type = type
             self[-1].title = ''
 
+# }}}
 
 class MetadataField(object):
 
-    def __init__(self, name, is_dc=True, formatter=None, none_is=None):
+    def __init__(self, name, is_dc=True, formatter=None, none_is=None,
+            renderer=lambda x: unicode(x)):
         self.name      = name
         self.is_dc     = is_dc
         self.formatter = formatter
         self.none_is   = none_is
+        self.renderer  = renderer
 
     def __real_get__(self, obj, type=None):
         ans = obj.get_metadata_element(self.name)
@@ -410,9 +420,31 @@ class MetadataField(object):
             return
         if elem is None:
             elem = obj.create_metadata_element(self.name, is_dc=self.is_dc)
-        obj.set_text(elem, unicode(val))
+        obj.set_text(elem, self.renderer(val))
+
+
+def serialize_user_metadata(metadata_elem, all_user_metadata, tail='\n'+(' '*8)):
+    from calibre.utils.config import to_json
+    from calibre.ebooks.metadata.book.json_codec import object_to_unicode
+
+    for name, fm in all_user_metadata.items():
+        try:
+            fm = object_to_unicode(fm)
+            fm = json.dumps(fm, default=to_json, ensure_ascii=False)
+        except:
+            prints('Failed to write user metadata:', name)
+            import traceback
+            traceback.print_exc()
+            continue
+        meta = metadata_elem.makeelement('meta')
+        meta.set('name', 'calibre:user_metadata:'+name)
+        meta.set('content', fm)
+        meta.tail = tail
+        metadata_elem.append(meta)
+
+
+class OPF(object): # {{{
 
-class OPF(object):
     MIMETYPE         = 'application/oebps-package+xml'
     PARSER           = etree.XMLParser(recover=True)
     NAMESPACES       = {
@@ -459,10 +491,11 @@ class OPF(object):
     series          = MetadataField('series', is_dc=False)
     series_index    = MetadataField('series_index', is_dc=False, formatter=float, none_is=1)
     rating          = MetadataField('rating', is_dc=False, formatter=int)
-    pubdate         = MetadataField('date', formatter=parse_date)
+    pubdate         = MetadataField('date', formatter=parse_date,
+                                    renderer=isoformat)
     publication_type = MetadataField('publication_type', is_dc=False)
     timestamp       = MetadataField('timestamp', is_dc=False,
-                                    formatter=parse_date)
+                                    formatter=parse_date, renderer=isoformat)
 
 
     def __init__(self, stream, basedir=os.getcwdu(), unquote_urls=True,
@@ -497,6 +530,43 @@ class OPF(object):
         self.guide = Guide.from_opf_guide(guide, basedir) if guide else None
         self.cover_data = (None, None)
         self.find_toc()
+        self.read_user_metadata()
+
+    def read_user_metadata(self):
+        self._user_metadata_ = {}
+        temp = Metadata('x', ['x'])
+        from calibre.utils.config import from_json
+        elems = self.root.xpath('//*[name() = "meta" and starts-with(@name,'
+                '"calibre:user_metadata:") and @content]')
+        for elem in elems:
+            name = elem.get('name')
+            name = ':'.join(name.split(':')[2:])
+            if not name or not name.startswith('#'):
+                continue
+            fm = elem.get('content')
+            try:
+                fm = json.loads(fm, object_hook=from_json)
+                temp.set_user_metadata(name, fm)
+            except:
+                prints('Failed to read user metadata:', name)
+                import traceback
+                traceback.print_exc()
+                continue
+            self._user_metadata_ = temp.get_all_user_metadata(True)
+
+    def to_book_metadata(self):
+        ans = MetaInformation(self)
+        for n, v in self._user_metadata_.items():
+            ans.set_user_metadata(n, v)
+        return ans
+
+    def write_user_metadata(self):
+        elems = self.root.xpath('//*[name() = "meta" and starts-with(@name,'
+                '"calibre:user_metadata:") and @content]')
+        for elem in elems:
+            elem.getparent().remove(elem)
+        serialize_user_metadata(self.metadata,
+                self._user_metadata_)
 
     def find_toc(self):
         self.toc = None
@@ -759,11 +829,10 @@ class OPF(object):
 
         def fset(self, val):
             matches = self.isbn_path(self.metadata)
-            if val is None:
-                if matches:
-                    for x in matches:
-                        x.getparent().remove(x)
-                    return
+            if not val:
+                for x in matches:
+                    x.getparent().remove(x)
+                return
             if not matches:
                 attrib = {'{%s}scheme'%self.NAMESPACES['opf']: 'ISBN'}
                 matches = [self.create_metadata_element('identifier',
@@ -911,6 +980,7 @@ class OPF(object):
         return elem
 
     def render(self, encoding='utf-8'):
+        self.write_user_metadata()
         raw = etree.tostring(self.root, encoding=encoding, pretty_print=True)
         if not raw.lstrip().startswith('<?xml '):
             raw = '<?xml version="1.0"  encoding="%s"?>\n'%encoding.upper()+raw
@@ -919,23 +989,30 @@ class OPF(object):
     def smart_update(self, mi, replace_metadata=False):
         for attr in ('title', 'authors', 'author_sort', 'title_sort',
                      'publisher', 'series', 'series_index', 'rating',
-                     'isbn', 'language', 'tags', 'category', 'comments',
+                     'isbn', 'tags', 'category', 'comments',
                      'pubdate'):
             val = getattr(mi, attr, None)
             if val is not None and val != [] and val != (None, None):
                 setattr(self, attr, val)
+        lang = getattr(mi, 'language', None)
+        if lang and lang != 'und':
+            self.language = lang
+        temp = self.to_book_metadata()
+        temp.smart_update(mi, replace_metadata=replace_metadata)
+        self._user_metadata_ = temp.get_all_user_metadata(True)
 
+# }}}
 
-class OPFCreator(MetaInformation):
+class OPFCreator(Metadata):
 
-    def __init__(self, base_path, *args, **kwargs):
+    def __init__(self, base_path, other):
         '''
         Initialize.
         @param base_path: An absolute path to the directory in which this OPF file
         will eventually be. This is used by the L{create_manifest} method
         to convert paths to files into relative paths.
         '''
-        MetaInformation.__init__(self, *args, **kwargs)
+        Metadata.__init__(self, title='', other=other)
         self.base_path = os.path.abspath(base_path)
         if self.application_id is None:
             self.application_id = str(uuid.uuid4())
@@ -1115,6 +1192,8 @@ class OPFCreator(MetaInformation):
                     item.set('title', ref.title)
                 guide.append(item)
 
+        serialize_user_metadata(metadata, self.get_all_user_metadata(False))
+
         root = E.package(
                 metadata,
                 manifest,
@@ -1156,7 +1235,7 @@ def metadata_to_opf(mi, as_string=True):
         <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
             <dc:identifier opf:scheme="%(a)s" id="%(a)s_id">%(id)s</dc:identifier>
             <dc:identifier opf:scheme="uuid" id="uuid_id">%(uuid)s</dc:identifier>
-        </metadata>
+            </metadata>
         <guide/>
     </package>
     '''%dict(a=__appname__, id=mi.application_id, uuid=mi.uuid)))
@@ -1188,7 +1267,7 @@ def metadata_to_opf(mi, as_string=True):
     factory(DC('contributor'), mi.book_producer, __appname__, 'bkp')
     if hasattr(mi.pubdate, 'isoformat'):
         factory(DC('date'), isoformat(mi.pubdate))
-    if mi.category:
+    if hasattr(mi, 'category') and mi.category:
         factory(DC('type'), mi.category)
     if mi.comments:
         factory(DC('description'), mi.comments)
@@ -1217,6 +1296,8 @@ def metadata_to_opf(mi, as_string=True):
     if mi.title_sort:
         meta('title_sort', mi.title_sort)
 
+    serialize_user_metadata(metadata, mi.get_all_user_metadata(False))
+
     metadata[-1].tail = '\n' +(' '*4)
 
     if mi.cover:
@@ -1334,5 +1415,30 @@ def suite():
 def test():
     unittest.TextTestRunner(verbosity=2).run(suite())
 
+def test_user_metadata():
+    from cStringIO import StringIO
+    mi = Metadata('Test title', ['test author1', 'test author2'])
+    um = {
+        '#myseries': { '#value#': u'test series\xe4', 'datatype':'text',
+            'is_multiple': None, 'name': u'My Series'},
+        '#myseries_index': { '#value#': 2.45, 'datatype': 'float',
+            'is_multiple': None},
+        '#mytags': {'#value#':['t1','t2','t3'], 'datatype':'text',
+            'is_multiple': '|', 'name': u'My Tags'}
+        }
+    mi.set_all_user_metadata(um)
+    raw = metadata_to_opf(mi)
+    opfc = OPFCreator(os.getcwd(), other=mi)
+    out = StringIO()
+    opfc.render(out)
+    raw2 = out.getvalue()
+    f = StringIO(raw)
+    opf = OPF(f)
+    f2 = StringIO(raw2)
+    opf2 = OPF(f2)
+    assert um == opf._user_metadata_
+    assert um == opf2._user_metadata_
+    print opf.render()
+
 if __name__ == '__main__':
-    test()
+    test_user_metadata()
