@@ -162,15 +162,6 @@ class FieldMetadata(dict):
                            'search_terms':['tags', 'tag'],
                            'is_custom':False,
                            'is_category':True}),
-            ('all_metadata',{'table':None,
-                             'column':None,
-                             'datatype':None,
-                             'is_multiple':None,
-                             'kind':'field',
-                             'name':None,
-                             'search_terms':[],
-                             'is_custom':False,
-                             'is_category':False}),
             ('author_sort',{'table':None,
                             'column':None,
                             'datatype':'text',
@@ -180,6 +171,15 @@ class FieldMetadata(dict):
                            'search_terms':['author_sort'],
                            'is_custom':False,
                            'is_category':False}),
+            ('au_map',    {'table':None,
+                           'column':None,
+                           'datatype':'text',
+                           'is_multiple':',',
+                           'kind':'field',
+                           'name':None,
+                           'search_terms':[],
+                           'is_custom':False,
+                           'is_category':False}),
             ('comments',  {'table':None,
                            'column':None,
                            'datatype':'text',
@@ -400,6 +400,12 @@ class FieldMetadata(dict):
         for key in self._tb_cats:
             yield (key, self._tb_cats[key])
 
+    def custom_iteritems(self):
+        for key in self._tb_cats:
+            fm = self._tb_cats[key]
+            if fm['is_custom']:
+                yield (key, self._tb_cats[key])
+
     def items(self):
         return list(self.iteritems())
 
